@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2015 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -91,11 +91,11 @@ def _rewrite_password(cmd, user=None, password=None):
     @rtype: C{str}
 
     """
-    # if there is a ,,-u X2GO_USER'' parameter in RDP options then we will replace 
+    # if there is a ,,-u X2GO_USER'' parameter in RDP options then we will replace
     # it by our X2Go session password
     if cmd and user:
         cmd = cmd.replace('X2GO_USER', user)
-    # if there is a ,,-p X2GO_PASSWORD'' parameter in RDP options then we will replace 
+    # if there is a ,,-p X2GO_PASSWORD'' parameter in RDP options then we will replace
     # it by our X2Go session password
     if cmd and password:
         cmd = cmd.replace('X2GO_PASSWORD', password)
@@ -324,7 +324,8 @@ class X2GoControlSession(paramiko.SSHClient):
                 timer.cancel()
 
             self.sftp_client = None
-        self._transport_lock.release()
+        if self._transport_lock.locked():
+            self._transport_lock.release()
 
     def _x2go_sftp_write(self, remote_path, content, timeout=20):
         """
@@ -380,7 +381,8 @@ class X2GoControlSession(paramiko.SSHClient):
                 timer.cancel()
 
             self.sftp_client = None
-        self._transport_lock.release()
+        if self._transport_lock.locked():
+            self._transport_lock.release()
 
     def _x2go_sftp_remove(self, remote_path, timeout=20):
         """
@@ -431,7 +433,8 @@ class X2GoControlSession(paramiko.SSHClient):
                 timer.cancel()
 
             self.sftp_client = None
-        self._transport_lock.release()
+        if self._transport_lock.locked():
+            self._transport_lock.release()
 
     def _x2go_exec_command(self, cmd_line, loglevel=log.loglevel_INFO, timeout=20, **kwargs):
         """
@@ -520,7 +523,8 @@ class X2GoControlSession(paramiko.SSHClient):
             self._transport_lock.release()
             raise x2go_exceptions.X2GoControlSessionException('the X2Go control session is not connected (while issuing SSH command=%s)' % cmd)
 
-        self._transport_lock.release()
+        if self._transport_lock.locked():
+            self._transport_lock.release()
 
         # sanitized X2Go relevant data, protect against data injection via .bashrc files
         (_stdin, _stdout, _stderr) = _retval
@@ -529,7 +533,7 @@ class X2GoControlSession(paramiko.SSHClient):
         sanitized_stdout = ''
         is_x2go_data = False
         for line in raw_stdout.split('\n'):
-            if line.startswith('X2GODATABEGIN:'+cmd_uuid): 
+            if line.startswith('X2GODATABEGIN:'+cmd_uuid):
                 is_x2go_data = True
                 continue
             if not is_x2go_data: continue
@@ -657,7 +661,7 @@ class X2GoControlSession(paramiko.SSHClient):
         ###
         ### FIXME:
         ###
-        # discussion about server-side access restriction based on posix group membership or similar currently 
+        # discussion about server-side access restriction based on posix group membership or similar currently
         # in process (as of 20110517, mg)
         #return username in self._x2go_remote_group('x2gousers')
         return True
@@ -773,7 +777,7 @@ class X2GoControlSession(paramiko.SSHClient):
         C{force_password_auth}, C{session_instance} and all SSH proxy related parameters
         have been added as X2Go specific parameters
 
-        The server's host key is checked against the system host keys 
+        The server's host key is checked against the system host keys
         (see C{load_system_host_keys}) and any local host keys (C{load_host_keys}).
         If the server's hostname is not found in either set of host keys, the missing host
         key policy is used (see C{set_missing_host_key_policy}).  The default policy is
@@ -821,11 +825,11 @@ class X2GoControlSession(paramiko.SSHClient):
         @param allow_agent: set to C{True} to enable connecting to a local SSH agent
             for acquiring authentication information
         @type allow_agent: C{bool}
-        @param add_to_known_hosts: non-paramiko option, if C{True} paramiko.AutoAddPolicy() 
-            is used as missing-host-key-policy. If set to C{False} paramiko.RejectPolicy() 
+        @param add_to_known_hosts: non-paramiko option, if C{True} paramiko.AutoAddPolicy()
+            is used as missing-host-key-policy. If set to C{False} paramiko.RejectPolicy()
             is used
         @type add_to_known_hosts: C{bool}
-        @param force_password_auth: non-paramiko option, disable pub/priv key authentication 
+        @param force_password_auth: non-paramiko option, disable pub/priv key authentication
             completely, even if the C{pkey} or the C{key_filename} parameter is given
         @type force_password_auth: C{bool}
         @param session_instance: an instance L{X2GoSession} using this L{X2GoControlSession}
@@ -873,7 +877,7 @@ class X2GoControlSession(paramiko.SSHClient):
         @raise X2GoSSHProxyException: any SSH proxy exception is passed through while establishing the SSH proxy connection and tunneling setup
         @raise X2GoSSHAuthenticationException: any SSH proxy authentication exception is passed through while establishing the SSH proxy connection and tunneling setup
         @raise X2GoRemoteHomeException: if the remote home directory does not exist or is not accessible
-        @raise X2GoControlSessionException: if the remote peer has died event unexpectedly
+        @raise X2GoControlSessionException: if the remote peer has died unexpectedly
 
         """
         _fake_hostname = None
@@ -894,7 +898,7 @@ class X2GoControlSession(paramiko.SSHClient):
 
         if unique_hostkey_aliases is None:
             unique_hostkey_aliases = self.unique_hostkey_aliases
-        # prep the fake hostname with the real hostname, so we trigger the corresponding code path in 
+        # prep the fake hostname with the real hostname, so we trigger the corresponding code path in
         # x2go.checkhosts and either of its missing host key policies
         if unique_hostkey_aliases:
             if port != 22: _fake_hostname = "[%s]:%s" % (hostname, port)
@@ -949,7 +953,7 @@ class X2GoControlSession(paramiko.SSHClient):
             if self.sshproxy_session is not None:
                 self.sshproxy_session.start()
 
-                # divert port to sshproxy_session's local forwarding port (it might have changed due to 
+                # divert port to sshproxy_session's local forwarding port (it might have changed due to
                 # SSH connection errors
                 gevent.sleep(.1)
                 port = self.sshproxy_session.get_local_proxy_port()
@@ -986,7 +990,7 @@ class X2GoControlSession(paramiko.SSHClient):
             try:
                 if password and force_password_auth:
                     self.logger('trying password based SSH authentication with server', loglevel=log.loglevel_DEBUG)
-                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, pkey=None, 
+                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, pkey=None,
                                                key_filename=None, timeout=timeout, allow_agent=False,
                                                look_for_keys=False)
                 elif (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
@@ -1204,7 +1208,8 @@ class X2GoControlSession(paramiko.SSHClient):
         self._session_auth_rsakey = None
 
         # in any case, release out internal transport lock
-        self._transport_lock.release()
+        if self._transport_lock.locked():
+            self._transport_lock.release()
 
         # close SSH agent auth forwarding objects
         if self.agent_handler is not None:
@@ -1279,7 +1284,7 @@ class X2GoControlSession(paramiko.SSHClient):
         """\
         Retrieve the menu tree of published applications from the remote X2Go server.
 
-        The C{raw} option lets this method return a C{list} of C{dict} elements. Each C{dict} elements has a 
+        The C{raw} option lets this method return a C{list} of C{dict} elements. Each C{dict} elements has a
         C{desktop} key containing a shortened version of the text output of a .desktop file and an C{icon} key
         which contains the desktop base64-encoded icon data.
 
@@ -1495,7 +1500,7 @@ class X2GoControlSession(paramiko.SSHClient):
 
     def resume(self, session_name=None, session_instance=None, session_list=None, **kwargs):
         """\
-        Resume a running/suspended X2Go session. 
+        Resume a running/suspended X2Go session.
 
         The L{X2GoControlSession.resume()} method accepts any parameter
         that can be passed to any of the C{X2GoTerminalSession*} backend class constructors.
@@ -1594,10 +1599,10 @@ class X2GoControlSession(paramiko.SSHClient):
 
     def list_desktops(self, raw=False, maxwait=20):
         """\
-        List all desktop-like sessions of current user (or of users that have 
+        List all desktop-like sessions of current user (or of users that have
         granted desktop sharing) on the connected server.
 
-        @param raw: if C{True}, the raw output of the server-side X2Go command 
+        @param raw: if C{True}, the raw output of the server-side X2Go command
             C{x2golistdesktops} is returned.
         @type raw: C{bool}
 
@@ -1628,7 +1633,7 @@ class X2GoControlSession(paramiko.SSHClient):
                 _listdesktops = _stdout_read.split('\n')
             except gevent.timeout.Timeout:
                 # if we do not get a reply here after <maxwait> seconds we will raise a time out, we have to
-                # make sure that we catch this at places where we want to ignore timeouts (e.g. in the 
+                # make sure that we catch this at places where we want to ignore timeouts (e.g. in the
                 # desktop list cache)
                 raise x2go_exceptions.X2GoTimeOutException('x2golistdesktop command timed out')
             finally:
@@ -1642,7 +1647,7 @@ class X2GoControlSession(paramiko.SSHClient):
 
         @param session_name: name of a session to query a list of mounts for
         @type session_name: C{str}
-        @param raw: if C{True}, the raw output of the server-side X2Go command 
+        @param raw: if C{True}, the raw output of the server-side X2Go command
             C{x2golistmounts} is returned.
         @type raw: C{bool}
         @param maxwait: stop processing C{x2golistmounts} after C{<maxwait>} seconds
@@ -1686,7 +1691,7 @@ class X2GoControlSession(paramiko.SSHClient):
         """\
         List all sessions of current user on the connected server.
 
-        @param raw: if C{True}, the raw output of the server-side X2Go command 
+        @param raw: if C{True}, the raw output of the server-side X2Go command
             C{x2golistsessions} is returned.
         @type raw: C{bool}
 
