@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2015 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -20,7 +20,7 @@
 """\
 X2GoTerminalSession class - core functions for handling your individual X2Go sessions.
 
-This backend handles X2Go server implementations that respond with session infos 
+This backend handles X2Go server implementations that respond with session infos
 via server-side PLAIN text output.
 
 """
@@ -127,8 +127,8 @@ class X2GoSessionParams(object):
         Rewrite the X2Go session type, so that the X2Go server
         can understand it (C{desktop} -> C{D}, etc.).
 
-        Also if the object's C{command} property is a known window 
-        manager, the session type will be set to 'D' 
+        Also if the object's C{command} property is a known window
+        manager, the session type will be set to 'D'
         (i.e. desktop).
 
         @return: 'D' if session should probably a desktop session,
@@ -171,7 +171,7 @@ class X2GoSessionParams(object):
         the passed on dictionary.
 
         @param properties_to_be_updated: a dictionary with L{X2GoSessionParams}
-            property names as keys und their values to be update in 
+            property names as keys und their values to be update in
             L{X2GoSessionParams} object.
         @type properties_to_be_updated: C{dict}
 
@@ -185,16 +185,16 @@ class X2GoTerminalSession(object):
     """\
     Class for managing X2Go terminal sessions on a remote X2Go server via Paramiko/SSH.
 
-    With the L{x2go.backends.terminal.plain.X2GoTerminalSession} class you can start new X2Go sessions, resume suspended 
-    sessions or suspend resp. terminate currently running sessions on a 
+    With the L{x2go.backends.terminal.plain.X2GoTerminalSession} class you can start new X2Go sessions, resume suspended
+    sessions or suspend resp. terminate currently running sessions on a
     connected X2Go server.
 
     An L{x2go.backends.terminal.plain.X2GoTerminalSession} object uses two main data structure classes:
 
-        - L{X2GoSessionParams}: stores all parameters that have been passed to the 
+        - L{X2GoSessionParams}: stores all parameters that have been passed to the
         constructor method.
 
-        - C{X2GoServerSessionInfo*} backend class: when starting or resuming a session, an object of this class 
+        - C{X2GoServerSessionInfo*} backend class: when starting or resuming a session, an object of this class
         will be used to store all information retrieved from the X2Go server.
 
     The terminal session instance works closely together (i.e. depends on) a connected control
@@ -240,8 +240,8 @@ class X2GoTerminalSession(object):
         @type pack: C{str}
         @param dpi: dots-per-inch value for the session screen (has an impact on the font size on screen)
         @type dpi: C{str}
-        @param cache_type: a dummy parameter that is passed to the L{x2go.backends.proxy.base.X2GoProxy}. In NX Proxy 
-            (class C{X2GoProxyNX3}) this originally is the session name. With X2Go it 
+        @param cache_type: a dummy parameter that is passed to the L{x2go.backends.proxy.base.X2GoProxy}. In NX Proxy
+            (class C{X2GoProxyNX3}) this originally is the session name. With X2Go it
             defines the name of the NX cache directory. Best is to leave it untouched.
         @type cache_type: C{str}
         @param kbtype: keyboard type, e.g. C{pc105/us} (default), C{pc105/de}, ...
@@ -255,7 +255,7 @@ class X2GoTerminalSession(object):
         @type clipboard: C{str}
         @param session_type: either C{desktop}, C{application} (rootless session) or C{shared}
         @type session_type: C{str}
-        @param snd_system: sound system to be used on server (C{none}, C{pulse} (default), 
+        @param snd_system: sound system to be used on server (C{none}, C{pulse} (default),
             C{arts} (obsolete) or C{esd})
         @type snd_system: C{str}
         @param snd_port: local sound port for network capable audio system
@@ -421,7 +421,8 @@ class X2GoTerminalSession(object):
           - clear the session info
 
         """
-        self._share_local_folder_lock.release()
+        if self._share_local_folder_lock.locked():
+            self._share_local_folder_lock.release()
         self.release_telekinesis()
         self.release_proxy()
         self.session_window = None
@@ -554,9 +555,9 @@ class X2GoTerminalSession(object):
                     self.control_session._x2go_sftp_put(local_path=cookie_filepath, remote_path='%s/.pulse-cookie' % self.session_info.remote_container)
 
                     # start reverse SSH tunnel for pulse stream
-                    _tunnel = rforward.X2GoRevFwTunnel(server_port=self.session_info.snd_port, 
-                                                       remote_host='127.0.0.1', 
-                                                       remote_port=self.snd_port, 
+                    _tunnel = rforward.X2GoRevFwTunnel(server_port=self.session_info.snd_port,
+                                                       remote_host='127.0.0.1',
+                                                       remote_port=self.snd_port,
                                                        ssh_transport=self.control_session.get_transport(),
                                                        session_instance=self.session_instance,
                                                        logger=self.logger
@@ -579,9 +580,9 @@ class X2GoTerminalSession(object):
                 self.control_session._x2go_sftp_put(local_path='%s/.esd_auth' % _LOCAL_HOME, remote_path='%s/.esd_auth' % self.control_session._x2go_remote_home)
 
                 # start reverse SSH tunnel for pulse stream
-                _tunnel = rforward.X2GoRevFwTunnel(server_port=self.session_info.snd_port, 
-                                                   remote_host='127.0.0.1', 
-                                                   remote_port=self.snd_port, 
+                _tunnel = rforward.X2GoRevFwTunnel(server_port=self.session_info.snd_port,
+                                                   remote_host='127.0.0.1',
+                                                   remote_port=self.snd_port,
                                                    ssh_transport=self.control_session.get_transport(),
                                                    session_instance=self.session_instance,
                                                    logger=self.logger
@@ -670,7 +671,7 @@ class X2GoTerminalSession(object):
         self.print_queue = printqueue.X2GoPrintQueue(profile_name=self.profile_name,
                                                      session_name=self.session_info.name,
                                                      spool_dir=spool_dir,
-                                                     print_action=self.print_action, 
+                                                     print_action=self.print_action,
                                                      print_action_args=self.print_action_args,
                                                      client_instance=self.client_instance,
                                                      printing_backend=self.printing_backend,
@@ -823,10 +824,10 @@ class X2GoTerminalSession(object):
         """\
         Share a local folder with the X2Go session.
 
-        @param local_path: the full path to an existing folder on the local 
+        @param local_path: the full path to an existing folder on the local
             file system
         @type local_path: C{str}
-        @param folder_type: one of 'disk' (a folder on your local hard drive), 'rm' (removeable device), 
+        @param folder_type: one of 'disk' (a folder on your local hard drive), 'rm' (removeable device),
             'cdrom' (CD/DVD Rom) or 'spool' (for X2Go print spooling)
         @type folder_type: C{str}
 
@@ -918,7 +919,7 @@ class X2GoTerminalSession(object):
                              '\'%s\'' % _CURRENT_LOCAL_USER,
                              _x2go_key_fname,
                              '%s__PRINT_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
-                             'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
+                             'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname),
                            ]
 
             elif folder_type == 'mimebox':
@@ -930,7 +931,7 @@ class X2GoTerminalSession(object):
                              '\'%s\'' % _CURRENT_LOCAL_USER,
                              _x2go_key_fname,
                              '%s__MIMEBOX_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
-                             'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
+                             'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname),
                            ]
 
             (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
@@ -961,7 +962,7 @@ class X2GoTerminalSession(object):
         self.logger('unsharing all local folders from session %s' % self.session_info, log.loglevel_INFO)
 
         cmd_line = [ 'export HOSTNAME &&',
-                     'x2goumount-session', 
+                     'x2goumount-session',
                      self.session_info.name,
                    ]
 
@@ -984,7 +985,7 @@ class X2GoTerminalSession(object):
         self.logger('unsharing local folder from session %s' % self.session_info, log.loglevel_INFO)
 
         cmd_line = [ 'export HOSTNAME &&',
-                     'x2goumount-session', 
+                     'x2goumount-session',
                      self.session_info.name,
                      "'%s'" % local_path,
                    ]
@@ -1241,7 +1242,7 @@ class X2GoTerminalSession(object):
         """\
         Run a command in this session.
 
-        After L{x2go.backends.terminal.plain.X2GoTerminalSession.start()} has been called 
+        After L{x2go.backends.terminal.plain.X2GoTerminalSession.start()} has been called
         one or more commands can be executed with L{x2go.backends.terminal.plain.X2GoTerminalSession.run_command()}
         within the current X2Go session.
 
@@ -1280,10 +1281,10 @@ class X2GoTerminalSession(object):
         if '/' in cmd:
             cmd = os.path.basename(cmd)
 
-        cmd_line = [ "setsid x2goruncommand", 
+        cmd_line = [ "setsid x2goruncommand",
                      str(self.session_info.display),
                      str(self.session_info.agent_pid),
-                     str(self.session_info.name), 
+                     str(self.session_info.name),
                      str(self.session_info.snd_port),
                      _rewrite_blanks(_rewrite_cmd(cmd, params=self.params)),
                      str(self.params.snd_system),
@@ -1437,7 +1438,7 @@ class X2GoTerminalSession(object):
         """\
         X2Go session connected?
 
-        @return: Returns C{True} if this X2Go session's Paramiko/SSH transport is 
+        @return: Returns C{True} if this X2Go session's Paramiko/SSH transport is
             connected/authenticated, C{False} else.
         @rtype: C{bool}
 
@@ -1531,7 +1532,7 @@ class X2GoTerminalSession(object):
                                                                )
 
         # set up SSH tunnel for X11 graphical elements
-        self.proxy = self.proxy_backend(session_info=self.session_info, 
+        self.proxy = self.proxy_backend(session_info=self.session_info,
                                         ssh_transport=self.control_session.get_transport(),
                                         sessions_rootdir=self.sessions_rootdir,
                                         session_instance=self.session_instance,
@@ -1557,7 +1558,7 @@ class X2GoTerminalSession(object):
 
     def resume(self):
         """\
-        Resume a running/suspended X2Go session. 
+        Resume a running/suspended X2Go session.
 
         @return: C{True} if the session could successfully be resumed
         @rtype: C{bool}
@@ -1620,11 +1621,11 @@ class X2GoTerminalSession(object):
         # local path may be a Windows path, so we use the path separator of the local system
         self.session_info.local_container = os.path.join(self.params.rootdir, 'S-%s' % self.session_info.name)
         # remote path is always a UniX path...
-        self.session_info.remote_container = '%s/.x2go/C-%s' % (self.control_session._x2go_remote_home, 
+        self.session_info.remote_container = '%s/.x2go/C-%s' % (self.control_session._x2go_remote_home,
                                                                 self.session_info.name,
                                                                )
-        self.proxy = self.proxy_backend(session_info=self.session_info, 
-                                        ssh_transport=self.control_session.get_transport(), 
+        self.proxy = self.proxy_backend(session_info=self.session_info,
+                                        ssh_transport=self.control_session.get_transport(),
                                         sessions_rootdir=self.sessions_rootdir,
                                         session_instance=self.session_instance,
                                         proxy_options=self.proxy_options,
